package com.faraphel.tasks_valider.connectivity.task

import com.faraphel.tasks_valider.database.TaskDatabase
import com.faraphel.tasks_valider.database.api.base.BaseApi
import com.google.gson.Gson
import fi.iki.elonen.NanoHTTPD


/**
 * A server to handle the task API to allow clients to interact with the database.
 * @param port the port of the server
 * @param database the database to interact with
 */
class TaskServer(
    private val port: Int,
    private val database: TaskDatabase
) : NanoHTTPD(port) {
    private val jsonParser = Gson()

    override fun serve(session: IHTTPSession): Response {
        val method: Method = session.method
        val uri: String = session.uri

        // remove the first slash
        val entityName: String = uri.substring(1)
        // get the matching entity API in the database
        val entityApi = database.getApiFromName(entityName)
            ?: return newFixedLengthResponse(
                Response.Status.NOT_FOUND,
                "text/plain",
                "Invalid entity name"
            )

        // handle the request
        return when (method) {
            // check if the data is in the database
            Method.HEAD -> entityApi.head(session)
            // get the data from the dao
            Method.GET -> entityApi.get(session)
            // insert the data into the database
            Method.POST -> entityApi.post(session)
            // other methods are not allowed
            else ->
                newFixedLengthResponse(
                    Response.Status.METHOD_NOT_ALLOWED,
                    "text/plain",
                    "Method not allowed"
                )
            // TODO(Faraphel): implement a permission system
        }
    }

    /**
     * Start the server with the default configuration
     */
    override fun start() {
        super.start(SOCKET_READ_TIMEOUT, false)
    }
}
