package com.faraphel.tasks_valider.database

import androidx.room.Database
import androidx.room.RoomDatabase
import androidx.room.TypeConverters
import com.faraphel.tasks_valider.database.api.*
import com.faraphel.tasks_valider.database.api.base.BaseApi
import com.faraphel.tasks_valider.database.converters.InstantConverter
import com.faraphel.tasks_valider.database.dao.GroupDao
import com.faraphel.tasks_valider.database.dao.GroupStudentDao
import com.faraphel.tasks_valider.database.dao.StudentDao
import com.faraphel.tasks_valider.database.dao.TaskDao
import com.faraphel.tasks_valider.database.dao.TaskGroupDao
import com.faraphel.tasks_valider.database.dao.TeacherDao
import com.faraphel.tasks_valider.database.entities.GroupEntity
import com.faraphel.tasks_valider.database.entities.GroupStudentEntity
import com.faraphel.tasks_valider.database.entities.StudentEntity
import com.faraphel.tasks_valider.database.entities.TaskEntity
import com.faraphel.tasks_valider.database.entities.TaskGroupEntity
import com.faraphel.tasks_valider.database.entities.TeacherEntity


/**
 * The database for the tasks application.
 * Contains the entities and the relations between them.
 */
@Database(
    entities = [
        GroupEntity::class,
        StudentEntity::class,
        TeacherEntity::class,
        TaskEntity::class,

        GroupStudentEntity::class,
        TaskGroupEntity::class,
    ],
    version = 1
)
@TypeConverters(
    InstantConverter::class
)
abstract class TaskDatabase : RoomDatabase() {
    // entities

    abstract fun groupDao(): GroupDao
    abstract fun studentDao(): StudentDao
    abstract fun teacherDao(): TeacherDao
    abstract fun taskDao(): TaskDao

    abstract fun groupStudentDao(): GroupStudentDao
    abstract fun taskGroupDao(): TaskGroupDao

    // api

    private val api: Map<String, BaseApi> = mapOf(
        "group" to GroupApi(this.groupDao()),
        "student" to StudentApi(this.studentDao()),
        "teacher" to TeacherApi(this.teacherDao()),
        "task" to TaskApi(this.taskDao()),

        "group_student" to GroupStudentApi(this.groupStudentDao()),
        "task_group" to TaskGroupApi(this.taskGroupDao()),
    )

    /**
     * get the api for an entity from its name.
     * @param name the name of the entity
     */
    fun getApiFromName(name: String): BaseApi? {
        return this.api[name]
    }
}
